@@ -2,12 +2,10 @@ from __future__ import annotations
 
 import inspect
 import logging
-import os
 import sys
 import time
 import traceback
 import uuid
-from collections import KeysView
 from inspect import signature
 from textwrap import dedent
 from typing import List, Optional, Dict
@@ -16,11 +14,10 @@ from uuid import uuid4
 import yaml
 
 from kodexa.connectors import FolderConnector
-from kodexa.connectors.connectors import get_caller_dir, DocumentStoreConnector
-from kodexa.model import Document, Store, ContentObject
-from kodexa.stores.stores import DocumentStore
+from kodexa.connectors.connectors import get_caller_dir
+from kodexa.model import Document, ContentObject
 
-logger = logging.getLogger('kodexa.pipeline')
+logger = logging.getLogger()
 
 
 def new_id():
@@ -65,40 +62,6 @@ class InMemoryContentProvider:
         self.content_objects[content_object.id] = content
 
 
-class InMemoryStoreProvider:
-    """A store provider is used to support getting stores from the pipeline"""
-
-    def __init__(self):
-        self.stores = {}
-
-    def put_store(self, name: str, store: Store):
-        """
-
-        Args:
-          name: str:
-          store: Store:
-
-        Returns:
-
-        """
-        self.stores[name] = store
-
-    def get_store(self, name):
-        """
-
-        Args:
-          name:
-
-        Returns:
-
-        """
-        return self.stores[name] if name in self.stores else None
-
-    def get_store_names(self) -> KeysView:
-        """ """
-        return self.stores.keys()
-
-
 class PipelineContext:
     """Pipeline context is created when you create a pipeline and it provides a way to access information about the
     pipeline that is running.  It can be made available to steps/functions so they can interact with it.
@@ -111,13 +74,12 @@ class PipelineContext:
 
     """
 
-    def __init__(self, content_provider=None, store_provider=None,
+    def __init__(self, content_provider=None,
                  existing_content_objects=None,
-                 context=None, execution_id=None):
+                 context=None, execution_id=None,
+                 status_handler=None, cancellation_handler=None):
         if content_provider is None:
             content_provider = InMemoryContentProvider()
-        if store_provider is None:
-            store_provider = InMemoryStoreProvider()
         if context is None:
             context = {}
         if existing_content_objects is None:
@@ -129,12 +91,23 @@ class PipelineContext:
         self.content_objects: List[ContentObject] = existing_content_objects
         self.content_provider = content_provider
         self.context: Dict = context
-        self.store_provider = store_provider
         self.stop_on_exception = True
         self.current_document = None
         self.document_family = None
         self.content_object = None
         self.document_store = None
+        self.status_handler = status_handler
+        self.cancellation_handler = cancellation_handler
+
+    def update_status(self, status_message: str, status_full_message: Optional[str] = None):
+        if self.status_handler is not None:
+            self.status_handler(status_message, status_full_message)
+
+    def is_cancelled(self) -> bool:
+        if self.cancellation_handler is not None:
+            return self.cancellation_handler()
+
+        return False
 
     def get_context(self) -> Dict:
         """ """
@@ -167,30 +140,6 @@ class PipelineContext:
         """
         self.content_provider.put_content(content_object, content)
 
-    def add_store(self, name: str, store):
-        """Add a store with given name to the context
-
-        Args:
-          name: the name to refer to the store with
-          store: the instance of the store
-          name: str:
-
-        Returns:
-
-        """
-        self.store_provider.put_store(name, store)
-
-    def get_store_names(self) -> KeysView:
-        """
-
-        Args:
-
-        Returns:
-          :return: the list of store names
-
-        """
-        return self.store_provider.get_store_names()
-
     def set_current_document(self, current_document: Document):
         """Set the Document that is currently being processed in the pipeline
 
@@ -228,45 +177,6 @@ class PipelineContext:
         """
         self.output_document = output_document
 
-    def get_store(self, name: str, default: Store = None) -> Store:
-        """Get a store with given name from the context
-
-        Args:
-          name: the name to refer to the store with
-          default: optionally the default to create the store as if it isn't there
-          name: str:
-          default: Store:  (Default value = None)
-
-        Returns:
-          the store, or None is not available
-
-        """
-        store = self.store_provider.get_store(name) if name in self.get_store_names() else None
-
-        if not store and default:
-            self.store_provider.put_store(name, default)
-            default.set_pipeline_context(self)
-
-        return self.store_provider.get_store(name)
-
-    def merge_store(self, name, store):
-        """
-        Merge a store into the pipeline context
-
-        Args:
-          name:
-          store:
-
-        Returns:
-
-        """
-        if name not in self.get_store_names():
-            logger.debug("New store, adding")
-            self.add_store(name, store)
-        else:
-            logger.debug("Existing store, merging")
-            self.get_store(name).merge(store)
-
 
 class PipelineStep:
     """The representation of a step within a step, which captures both the step itself and
@@ -275,15 +185,13 @@ class PipelineStep:
     It is internally used by the Pipeline and is not a public API
     """
 
-    def __init__(self, step, enabled=False, name=None, options=None, attach_source=False,
-                 parameterized=False, step_type='ACTION'):
+    def __init__(self, step, name=None, options=None, attach_source=False,
+                 step_type='ACTION'):
         if options is None:
             options = {}
         self.step = step
         self.name = name
-        self.enabled = enabled
         self.options = options
-        self.parameterized = parameterized
         self.step_type = step_type
 
         if str(type(self.step)) == "<class 'type'>":
@@ -294,8 +202,8 @@ class PipelineStep:
             self.name = step.__name__
         elif isinstance(self.step, str):
             logger.info(f"Adding new remote step {step} to pipeline")
-            from kodexa import RemoteAction
-            self.step = RemoteAction(step, options=options, attach_source=attach_source)
+            from kodexa import RemoteStep
+            self.step = RemoteStep(step, step_type=step_type, options=options, attach_source=attach_source)
         else:
             logger.info(f"Adding new step {type(step)} to pipeline")
 
@@ -304,12 +212,12 @@ class PipelineStep:
         try:
             if str(type(self.step)) == "<class 'type'>":
                 raise Exception("You can not yet deploy a pipeline with a class instance style step")
-            elif isinstance(self.step, str):
+            if isinstance(self.step, str):
                 return {
                     'ref': self.step,
                     'options': self.options
                 }
-            elif callable(self.step):
+            if callable(self.step):
                 metadata = {
                     'function': self.step.__name__,
                     'script': dedent(inspect.getsource(self.step))
@@ -318,8 +226,6 @@ class PipelineStep:
                 metadata = self.step.to_dict()
 
             metadata['name'] = self.name
-            metadata['parameterized'] = self.parameterized
-            metadata['enabled'] = self.enabled
             metadata['stepType'] = self.step_type
             return metadata
         except AttributeError as e:
@@ -328,88 +234,54 @@ class PipelineStep:
     def execute(self, context, document):
 
         start = time.perf_counter()
-        if self.enabled:
-            try:
-
-                context.set_current_document(document)
-                logger.info(f"Starting step")
-                if str(type(self.step)) == "<class 'type'>":
-
-                    logger.info(f"Starting step based on class {self.step}")
-
-                    # We need to handle the parameterization
-                    import copy
-
-                    option_copy = copy.deepcopy(self.options)
-                    if self.parameterized:
-                        import collections
-
-                        def replace_params(opts, params):
-
-                            if isinstance(opts, dict):
-                                for key, val in opts.items():
-                                    opts[key] = replace_params(val, params)
-                            elif isinstance(opts, list):
-                                new_list = []
-                                for list_val in opts:
-                                    new_list.append(replace_params(list_val, params))
-                                return new_list
-                            elif isinstance(opts, str):
-                                if opts.startswith('${') and opts.endswith('}'):
-                                    param_name = opts[2:-1]
-                                    if param_name in params:
-                                        return params[param_name]
-                                    else:
-                                        raise Exception("Unable to find parameter name " + param_name)
-
-                            return opts
-
-                        replace_params(option_copy, context.parameters)
-
-                    step_instance = self.step(**option_copy)
-                    if len(signature(step_instance.process).parameters) == 1:
-                        result_document = step_instance.process(document)
-                    else:
-                        result_document = step_instance.process(document, context)
-
-                elif not callable(self.step):
-                    logger.info(f"Starting step {type(self.step)}")
-
-                    if len(signature(self.step.process).parameters) == 1:
-                        result_document = self.step.process(document)
-                    else:
-                        result_document = self.step.process(document, context)
-                else:
-                    logger.info(f"Starting step function {self.step.__name__}")
-
-                    if len(signature(self.step).parameters) == 1:
-                        result_document = self.step(document)
-                    else:
-                        result_document = self.step(document, context)
-
-                end = time.perf_counter()
-                logger.info(f"Step completed (f{end - start:0.4f}s)")
-
-                return result_document
-            except:
-                logger.warning("Step failed")
-                tt, value, tb = sys.exc_info()
-                document.exceptions.append({
-                    "step": self.step.__name__ if callable(self.step) else type(self.step),
-                    "traceback": traceback.format_exception(tt, value, tb)
-                })
-                if context.stop_on_exception:
-                    raise
-                else:
-                    return document
-        else:
-            return document
-
-    def end_processing(self, context):
         try:
-            self.step.end_processing(context)
+
+            context.set_current_document(document)
+            logger.info(f"Starting step")
+            if str(type(self.step)) == "<class 'type'>":
+
+                logger.info(f"Starting step based on class {self.step}")
+
+                # We need to handle the parameterization
+                import copy
+
+                option_copy = copy.deepcopy(self.options)
+                step_instance = self.step(**option_copy)
+                if len(signature(step_instance.process).parameters) == 1:
+                    result_document = step_instance.process(document)
+                else:
+                    result_document = step_instance.process(document, context)
+
+            elif not callable(self.step):
+                logger.info(f"Starting step {type(self.step)}")
+
+                if len(signature(self.step.process).parameters) == 1:
+                    result_document = self.step.process(document)
+                else:
+                    result_document = self.step.process(document, context)
+            else:
+                logger.info(f"Starting step function {self.step.__name__}")
+
+                if len(signature(self.step).parameters) == 1:
+                    result_document = self.step(document)
+                else:
+                    result_document = self.step(document, context)
+
+            end = time.perf_counter()
+            logger.info(f"Step completed (f{end - start:0.4f}s)")
+
+            return result_document
         except:
-            pass
+            logger.warning("Step failed")
+            tt, value, tb = sys.exc_info()
+            document.exceptions.append({
+                "step": self.step.__name__ if callable(self.step) else type(self.step),
+                "traceback": traceback.format_exception(tt, value, tb)
+            })
+            if context.stop_on_exception:
+                raise
+
+            return document
 
 
 class LabelStep(object):
@@ -419,10 +291,6 @@ class LabelStep(object):
         self.label = label
         self.remove = remove
 
-    def get_name(self):
-        """ """
-        return f"Remove label {self.label}" if self.remove else f"Add label {self.label}"
-
     def process(self, document: Document):
         """
 
@@ -439,27 +307,6 @@ class LabelStep(object):
         return document
 
 
-class PipelineStore:
-    """ """
-
-    def __init__(self, name: str, store: Store, extracted_labelled: bool = False):
-        self.name = name
-        self.store = store
-        self.extract_labelled = extracted_labelled
-
-    def extract(self, document):
-        """
-
-        Args:
-          document:
-
-        Returns:
-
-        """
-        # TODO implement
-        pass
-
-
 class Pipeline:
     """A pipeline represents a way to bring together parts of the kodexa framework to solve a specific problem.
 
@@ -487,41 +334,18 @@ class Pipeline:
             self.connector = connector
 
         self.steps: List[PipelineStep] = []
-        self.stores: List[PipelineStore] = []
         self.name = name
         self.stop_on_exception = stop_on_exception
         self.logging_level = logging_level
         self.apply_lineage = apply_lineage
 
-    def add_store(self, name: str, store: Store, extracted_labelled=False):
-        """Add the store to the pipeline so that it is available to the pipeline
-
-        Args:
-          name: the name of the store (to refer to it)
-          store: the store that should be added
-          extracted_labelled: at the end of the pipeline we will extract the labelled data
-        to this store (Default value = False)
-          name: str:
-          store: Store:
-
-        Returns:
-
-        >>> pipeline = Pipeline(FolderConnector(path='/tmp/', file_filter='example.pdf'))
-            >>> pipeline.add_store("test-store", TableDataStore())
-        """
-        self.stores.append(PipelineStore(name, store, extracted_labelled))
-        return self
-
-    def add_label(self, label: str, enabled=True, condition=None, options=None, attach_source=False,
-                  parameterized=False, cache_path=None):
+    def add_label(self, label: str, options=None, attach_source=False):
         """Adds a label to the document
 
         Args:
           label: label to add
-          enabled: is the step enabled (default True)
           options: options to be passed to the step if it is a simplified remote action (Default value = None)
           attach_source: if step is simplified remote action this determines if we need to add the source (Default value = False)
-          parameterized: apply the pipeline's parameters to the options (Default value = False)
           label: str:
 
         Returns:
@@ -529,21 +353,18 @@ class Pipeline:
 
         """
         self.steps.append(
-            PipelineStep(step=LabelStep(label), name=f"Add label {label}", enabled=enabled,
+            PipelineStep(step=LabelStep(label), name=f"Add label {label}",
                          options=options,
-                         attach_source=attach_source, parameterized=parameterized))
+                         attach_source=attach_source))
         return self
 
-    def remove_label(self, label: str, enabled=True, options=None, attach_source=False,
-                     parameterized=False):
+    def remove_label(self, label: str, options=None, attach_source=False):
         """Adds a label to the document
 
         Args:
           label: label to remove
-          enabled: is the step enabled (default True)
           options: options to be passed to the step if it is a simplified remote action (Default value = None)
           attach_source: if step is simplified remote action this determines if we need to add the source (Default value = False)
-          parameterized: apply the pipeline's parameters to the options (Default value = False)
           label: str: the label to add
 
         Returns:
@@ -551,13 +372,13 @@ class Pipeline:
 
         """
         self.steps.append(
-            PipelineStep(step=LabelStep(label, remove=True), name=f"Remove label {label}", enabled=enabled,
+            PipelineStep(step=LabelStep(label, remove=True), name=f"Remove label {label}",
                          options=options,
-                         attach_source=attach_source, parameterized=parameterized))
+                         attach_source=attach_source))
         return self
 
-    def add_step(self, step, name=None, enabled=True, options=None, attach_source=False,
-                 parameterized=False, step_type='ACTION'):
+    def add_step(self, step, name=None, options=None, attach_source=False,
+                 step_type='ACTION'):
         """Add the given step to the current pipeline
 
 
@@ -570,10 +391,8 @@ class Pipeline:
         Args:
           step: the step to add
           name: the name to use to describe the step (default None)
-          enabled: is the step enabled (default True)
           options: options to be passed to the step if it is a simplified remote action (Default value = None)
           attach_source: if step is simplified remote action this determines if we need to add the source (Default value = False)
-          parameterized: apply the pipeline's parameters to the options (Default value = False)
           step_type: the type of step to add, can either be an ACTION or MODEL
         Returns:
           the instance of the pipeline
@@ -590,8 +409,8 @@ class Pipeline:
         """
         if options is None:
             options = {}
-        self.steps.append(PipelineStep(step=step, name=name, enabled=enabled, options=options,
-                                       attach_source=attach_source, parameterized=parameterized,
+        self.steps.append(PipelineStep(step=step, name=name, options=options,
+                                       attach_source=attach_source,
                                        step_type=step_type))
 
         return self
@@ -621,8 +440,7 @@ class Pipeline:
         return yaml.dump(configuration_steps)
 
     def run(self, parameters=None):
-        """Run the current pipeline, note that you must have a sink in place to allow the pipeline to run
-
+        """Run the current pipeline
 
         :return: The context from the run
 
@@ -632,7 +450,6 @@ class Pipeline:
         Returns:
 
         >>> pipeline = Pipeline(FolderConnector(path='/tmp/', file_filter='example.pdf'))
-            >>> pipeline.set_sink(ExampleSink())
             >>> pipeline.run()
         """
         if parameters is None:
@@ -644,10 +461,6 @@ class Pipeline:
         self.context = PipelineContext()
         self.context.stop_on_exception = self.stop_on_exception
 
-        for pipeline_store in self.stores:
-            logger.info(f"Adding store {pipeline_store.name}")
-            self.context.add_store(pipeline_store.name, pipeline_store.store)
-
         self.context.statistics = PipelineStatistics()
         self.context.parameters = parameters
 
@@ -693,40 +506,13 @@ class Pipeline:
                 self.context.statistics.processed_document(document)
                 self.context.output_document = document
 
-                # Determine if any of the stores will automatically extract the data
-                for store in self.stores:
-                    if store.extract_labelled:
-                        store.extract(document)
-
             else:
                 logger.warning("A step did not return a document?")
 
-        logger.info(f"Completing pipeline {self.name}")
-
-        for step in self.steps:
-            step.end_processing(self.context)
-
         logger.info(f"Completed pipeline {self.name}")
 
         return self.context
 
-    @staticmethod
-    def from_store(store: DocumentStore, subscription=None, *args, **kwargs):
-        """Build a new pipeline with the input documents from a document store
-
-        Args:
-          store: DocumentStore The URL ie. https://www.google.com
-          subscription: str The subscription query to use (Default value = None)
-          store: DocumentStore:
-          *args:
-          **kwargs:
-
-        Returns:
-          A new instance of a pipeline
-
-        """
-        return Pipeline(DocumentStoreConnector(store, subscription), *args, **kwargs)
-
     @staticmethod
     def from_url(url, headers=None, *args, **kwargs):
         """Build a new pipeline with the input being a document created from the given URL
@@ -817,7 +603,6 @@ class PipelineStatistics:
 
     def __init__(self):
         self.documents_processed = 0
-        self.document_exceptions = 0
 
     def processed_document(self, document):
         """Update statistics based on this document completing processing
@@ -829,6 +614,3 @@ class PipelineStatistics:
 
         """
         self.documents_processed += 1
-
-        if document and document.exceptions:
-            self.document_exceptions += 1
